#!/bin/sh

instance=$(jj -i config.json instance)
ordering='title'
default_player_command='mpv --no-vid --no-ytdl'

funkwhale_api_check_api_limits()
{
	curl -s "https://$instance/api/v1/rate-limit/" | jj -p
}

funkwhale_api_get_tracks()
{
	get_json=$(curl -s "https://$instance/api/v1/tracks?ordering=$ordering&playable=true&page=$1" 2>&1 | tee preload)
	jj -i preload -l 'results.#.uploads.0.listen_url' | sed 's/"//g'
}

funkwhale_api_get_tracks_from_channel()
{
	track_list=$(curl -s "https://$instance/api/v1/tracks?channel=$1&playable=true&include_channels=true")  
	echo "Loaded $(echo $track_list | jj count) podcasts"
	echo $track_list | jj -l results.#.title
	listen_urls=$(echo $track_list | jj -l results.#.listen_url | sed 's/"//g')
	echo > podcast.m3u8
	for i in $listen_urls; do
		echo "https://$instance$i\n" >> podcast.m3u8
	done
	$default_player_command podcast.m3u8
}

funkwhale_get_podcasts_artists()
{
	echo 'Loading podcast artists...'
	curl -s --output preload "https://$instance/api/v1/artists?ordering=-creation_date&playable=true&include_channels=true&content_category=podcast&page=$1"
	counter=0
	count_artists=$(jj -i preload count)
	echo "$count_artists avalaible"
	artist_channels=$(jj -i preload -l 'results.#.channel.uuid' | sed 's/"//g')
	artist_names=$(mktemp)
	temp_artists=$(mktemp)
	for i in $artist_channels; do
		jj -i preload results."$counter".name | sed 's/\./_/g' >> $artist_names
		jj -i $temp_artists -v "$i" -o $temp_artists "$(jj -i preload results.$counter.name | sed 's/\./_/g')"
		counter=$(expr $counter + 1)
	done
	menu_artist_choice=$(cat $artist_names | fzy)
	channel_uuid=$(jj -i $temp_artists "$menu_artist_choice")
	funkwhale_api_get_tracks_from_channel $channel_uuid
}

load_tracks_to_playlist()
{
	echo "Loading page $1 ..."
	echo '#EXTM3U\n' > playlist.m3u8
	counter_titles=0
	for i in $(funkwhale_api_get_tracks $1); do
		title=$(jj -i preload results."$counter_titles".title)
		echo "#EXTINF:-1, $counter_titles - $title" >> playlist.m3u8
		counter_titles=$(expr $counter_titles + 1)
		echo "https://$instance$i\n" >> playlist.m3u8
	done
	count_tracks_results=$(jj -i preload 'results.#')
	if [ -z $count_tracks_results ]; then
		echo 'Error: Page empty or 500 Internal Server Error'
	else
		echo "Loaded $count_tracks_results tracks"
	fi
}

get_all_avalaible_count_tracks() 
{
	funkwhale_api_get_tracks 1 1>> /dev/null
	count_all_tracks=$(jj -i preload -l count)
	echo "Tracks avalaible on $instance: $count_all_tracks\n"
}

get_all_avalaible_count_tracks

downloadtrackspls='Download tracks in playlist'
startplayer='Start player'
podcasts='Podcasts'
changepod='Switch instance'
checkapilimits='Check API limits (debug)'
Exit='Exit'

while true; do
choice=$(echo "$downloadtrackspls\n$startplayer\n$podcasts\n$changepod\n$checkapilimits\n$Exit" | fzy)

if [ "$choice" = "$downloadtrackspls" ]; then
	echo 'Order by (prefix - is DESC ordering):'
	ordering=$(echo 'title\n-title\ncreation_date\n-creation_date\nrelease_date\n-release_date\nrandom' | fzy)
	export ordering
	echo 'Enter page number: '
	read page
	load_tracks_to_playlist $page
elif [ "$choice" = "$startplayer" ]; then
	$default_player_command playlist.m3u8
elif [ "$choice" = "$podcasts" ]; then
	echo 'Enter page number: '
	read page
	funkwhale_get_podcasts_artists $page
elif [ "$choice" = "$changepod" ]; then
	echo "Type instance (ex. $instance):"
	read instance
	export instance
	conf_instance_state=$(echo 'Permanent\nTemporaly' | fzy)
	if [ "$conf_instance_state" = 'Permanent' ]; then
		jj -i config.json instance -v $instance -o config.json
	else
		echo ''
	fi
	clear
	get_all_avalaible_count_tracks
elif [ "$choice" = "$checkapilimits" ]; then
	funkwhale_api_check_api_limits | more
elif [ "$choice" = "$Exit" ]; then
	exit 0
fi
done
